#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.registry module

This module defines components which are used to display local and global registries contents.
"""

from html import escape

from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import MANAGE_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_skin.interfaces.viewlet import IHelpViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import get_local_registry, get_pyramid_registry
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IUtilitiesMenu
from pyams_zmi.table import NameColumn, Table, TableAdminView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_zmi import _


class RegistryTable(Table):
    """Base registry table"""


@adapter_config(name='component',
                required=(ISiteRoot, IAdminLayer, RegistryTable),
                provides=IColumn)
class RegistryTableComponentColumn(NameColumn):
    """Registry table component column"""

    i18n_header = _("Component")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 10

    def get_value(self, obj):
        """Component column value getter"""
        component = obj.component
        if component is not None:
            name = getattr(component, '__name__', None)
            if not name:
                name = str(component.__class__)
        else:
            name = str(obj.factory)
        return escape(name)


@adapter_config(name='interface',
                required=(ISiteRoot, IAdminLayer, RegistryTable),
                provides=IColumn)
class RegistryTableInterfaceColumn(NameColumn):
    """Registry table interface column"""

    i18n_header = _("Interface")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 20

    def get_value(self, obj):
        """Interface column value getter"""
        return escape(str(obj.provided))


@adapter_config(name='name',
                required=(ISiteRoot, IAdminLayer, RegistryTable),
                provides=IColumn)
class RegistryTableNameColumn(NameColumn):
    """Registry table name column"""

    i18n_header = _("Name")
    css_classes = {
        'td': 'text-nowrap'
    }
    weight = 30

    def get_value(self, obj):
        """Name column value getter"""
        return obj.name or '--'


#
# Local registry views
#

@viewlet_config(name='local-registry.menu', context=ISiteRoot, layer=IAdminLayer,
                manager=IUtilitiesMenu, weight=10,
                permission=MANAGE_SYSTEM_PERMISSION)
class LocalRegistryMenu(NavigationMenuItem):
    """Local registry menu"""

    label = _("Local registry")
    href = '#local-registry.html'


class LocalRegistryTable(RegistryTable):
    """Local registry table"""


@adapter_config(required=(ISiteRoot, IAdminLayer, LocalRegistryTable),
                provides=IValues)
class LocalRegistryTableValues(ContextRequestViewAdapter):
    """Local registry table values adapter"""

    @property
    def values(self):
        """Local registry table values getter"""
        yield from get_local_registry().registeredUtilities()


@pagelet_config(name='local-registry.html', context=ISiteRoot, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
class LocalRegistryView(TableAdminView):
    """Local registry view"""

    title = _("Site utilities")
    table_class = LocalRegistryTable
    table_label = _("Local registry")


@viewlet_config(name='local-registry.info', context=ISiteRoot, layer=IAdminLayer,
                view=LocalRegistryView, manager=IHelpViewletManager, weight=1)
class LocalRegistryInfo(AlertMessage):
    """Local registry info"""

    status = 'info'
    css_class = 'mx-2'
    _message = _("""The local registry is used to store 'local' utilities, whose configuration \
is stored into the ZODB. Some of these utilities have important settings which can define \
the whole application behaviour.
Some of these components are created automatically, while other ones are created by sites \
managers, like SQLAlchemy or ZODB connections.
""")


#
# Global registry views
#

@viewlet_config(name='global-registry.menu', context=ISiteRoot, layer=IAdminLayer,
                manager=IUtilitiesMenu, weight=20,
                permission=MANAGE_SYSTEM_PERMISSION)
class GlobalRegistryMenu(NavigationMenuItem):
    """Global registry menu"""

    label = _("Global registry")
    href = '#global-registry.html'


class GlobalRegistryTable(RegistryTable):
    """Global registry table"""


@adapter_config(required=(ISiteRoot, IAdminLayer, GlobalRegistryTable),
                provides=IValues)
class GlobalRegistryTableValues(ContextRequestViewAdapter):
    """Global registry table values adapter"""

    @property
    def values(self):
        """Global registry table values getter"""
        yield from get_pyramid_registry().registeredUtilities()


@pagelet_config(name='global-registry.html', context=ISiteRoot, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
class GlobalRegistryView(TableAdminView):
    """Global registry view"""

    title = _("Site utilities")
    table_class = GlobalRegistryTable
    table_label = _("Global registry")


@viewlet_config(name='global-registry.info', context=ISiteRoot, layer=IAdminLayer,
                view=GlobalRegistryView, manager=IHelpViewletManager, weight=1)
class GlobalRegistryInfo(AlertMessage):
    """Global registry info"""

    status = 'info'
    css_class = 'mx-2'
    _message = _("""The global registry is used to store utilities which are defined statically \
in Python code or using ZCML directives.
""")



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.configuration module

This module provides support for basic management interface configuration based on MyAMS
features.
"""

import pkg_resources
from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import Interface
from zope.schema.fieldproperty import FieldProperty

from pyams_file.property import FileProperty
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_security.interfaces.base import MANAGE_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteEtcTraverser, ISiteRoot
from pyams_utils.adapter import ContextAdapter, ContextRequestViewAdapter, adapter_config, \
    get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.interfaces.tales import ITALESExtension
from pyams_zmi.interfaces.configuration import IZMIConfiguration


__docformat__ = 'restructuredtext'


@factory_config(IZMIConfiguration)
class ZMIConfiguration(Persistent, Contained):
    """ZMI configuration class"""

    site_name = FieldProperty(IZMIConfiguration['site_name'])
    home_name = FieldProperty(IZMIConfiguration['home_name'])
    application_name = FieldProperty(IZMIConfiguration['application_name'])
    application_package = FieldProperty(IZMIConfiguration['application_package'])
    inner_package_name = FieldProperty(IZMIConfiguration['inner_package_name'])
    inner_package = FieldProperty(IZMIConfiguration['inner_package'])
    environment = FieldProperty(IZMIConfiguration['environment'])

    @property
    def version(self):
        """Get complete version string"""
        result = pkg_resources.get_distribution(self.application_package).version
        if self.inner_package and (self.inner_package != self.application_package):
            result = '{}<span class="mx-2">&ndash;</span>{} - {}'.format(
                result,
                self.inner_package_name,
                pkg_resources.get_distribution(self.inner_package).version)
        return result

    myams_bundle = FieldProperty(IZMIConfiguration['myams_bundle'])
    favicon = FileProperty(IZMIConfiguration['favicon'])

    include_header = FieldProperty(IZMIConfiguration['include_header'])
    fixed_header = FieldProperty(IZMIConfiguration['fixed_header'])
    logo = FileProperty(IZMIConfiguration['logo'])

    include_site_search = FieldProperty(IZMIConfiguration['include_site_search'])
    site_search_placeholder = FieldProperty(IZMIConfiguration['site_search_placeholder'])
    site_search_handler = FieldProperty(IZMIConfiguration['site_search_handler'])

    include_menus = FieldProperty(IZMIConfiguration['include_menus'])
    include_minify_button = FieldProperty(IZMIConfiguration['include_minify_button'])
    fixed_navigation = FieldProperty(IZMIConfiguration['fixed_navigation'])
    accordion_menus = FieldProperty(IZMIConfiguration['accordion_menus'])

    include_ribbon = FieldProperty(IZMIConfiguration['include_ribbon'])
    fixed_ribbon = FieldProperty(IZMIConfiguration['fixed_ribbon'])

    base_body_css_class = FieldProperty(IZMIConfiguration['base_body_css_class'])

    @property
    def body_css_class(self):
        """Body CSS class getter"""
        result = self.base_body_css_class or ''
        for attr in ('header', 'navigation', 'ribbon'):
            if getattr(self, 'fixed_' + attr, False):
                result += ' fixed-' + attr
        return result

    custom_stylesheet = FileProperty(IZMIConfiguration['custom_stylesheet'])
    custom_script = FileProperty(IZMIConfiguration['custom_script'])


@adapter_config(required=IZMIConfiguration, provides=IViewContextPermissionChecker)
class ZMIConfigurationPermissionChecker(ContextAdapter):
    """ZMI configuration permission checker"""

    edit_permission = MANAGE_SYSTEM_PERMISSION


ZMI_CONFIGURATION_KEY = 'pyams_zmi.configuration'
"""Annotations key used to store ZMI configuration"""


@adapter_config(required=ISiteRoot, provides=IZMIConfiguration)
def zmi_configuration_factory(context):
    """ZMI configuration factory"""
    return get_annotation_adapter(context, ZMI_CONFIGURATION_KEY, IZMIConfiguration,
                                  notify=False, name='++etc++zmi.configuration')


@adapter_config(name='zmi.configuration', required=ISiteRoot, provides=ISiteEtcTraverser)
def site_root_zmi_configuration_traverser(context):
    """Site root ++etc++zmi.configuration traverser extension"""
    return IZMIConfiguration(context, None)


@adapter_config(name='zmi.configuration',
                required=(Interface, Interface, Interface),
                provides=ITALESExtension)
class ZMIConfigurationTalesExtension(ContextRequestViewAdapter):
    """zmi.configuration TALES extension"""

    def render(self, context=None):  # pylint: disable=unused-argument
        """TALES extension renderer"""
        return IZMIConfiguration(self.request.root, None)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.viewlet.breadcrumb module

This module defines breadcrumbs adapter for management interface.
"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface
from zope.location import ILocation

from pyams_i18n.interfaces import II18n
from pyams_site.interfaces import ISiteRoot
from pyams_skin.interfaces.viewlet import IBreadcrumbItem
from pyams_skin.viewlet.breadcrumb import BreadcrumbItem
from pyams_utils.adapter import adapter_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.configuration import IZMIConfiguration


@adapter_config(required=(ILocation, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class AdminLayerBreadcrumbItem(BreadcrumbItem):
    """Admin layer breadcrumb ietm adapter"""

    view_name = 'admin'


@adapter_config(required=(ISiteRoot, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class SiteRootBreadcrumbItem(AdminLayerBreadcrumbItem):
    """Site root breadcrumb item adapter"""

    @property
    def label(self):
        """Label getter"""
        configuration = IZMIConfiguration(self.request.root)
        return II18n(configuration).query_attribute('home_name', request=self.request) or \
            configuration.site_name

    css_class = 'breadcrumb-item persistent'



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.metas module

Custom management interface metas headers.
"""

__docformat__ = 'restructuredtext'

from datetime import datetime

from pyramid.httpexceptions import HTTPNotFound
from pyramid.view import view_config
from zope.dublincore.interfaces import IZopeDublinCore
from zope.interface import Interface

from pyams_file.skin.view import FileView
from pyams_layer.interfaces import IPyAMSLayer
from pyams_skin.interfaces.metas import IHTMLContentMetas
from pyams_skin.metas import LinkMeta
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.timezone import tztime
from pyams_utils.url import absolute_url
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.configuration import IZMIConfiguration


THUMB_HREF_STR = '{}/++thumb++{}x{}?_={}'


@adapter_config(name='favicon',
                required=(Interface, IAdminLayer, Interface),
                provides=IHTMLContentMetas)
class IconMetasAdapter(ContextRequestViewAdapter):
    """Icon metas adapter"""

    weight = 20

    def get_metas(self):
        """Metas getter"""
        config = IZMIConfiguration(self.request.root, None)
        if (config is not None) and (config.favicon is not None):
            icon = config.favicon
            icon_url = absolute_url(icon, self.request)
            icon_size = icon.get_image_size()[0]  # pylint: disable=no-member
            dc = IZopeDublinCore(icon)  # pylint: disable=invalid-name
            timestamp = datetime.timestamp(tztime(dc.modified))
            for size in (180, 144, 114, 72, 32, 16):
                if icon_size >= size:
                    yield LinkMeta('apple-touch-icon',
                                   type=icon.content_type,  # pylint: disable=no-member
                                   href=THUMB_HREF_STR.format(icon_url, size, size, timestamp),
                                   sizes='{0}x{0}'.format(size))
            for size in (128, 124, 32):
                if icon_size >= size:
                    yield LinkMeta('icon',
                                   type=icon.content_type,  # pylint: disable=no-member
                                   href=THUMB_HREF_STR.format(icon_url, size, size, timestamp),
                                   sizes='{0}x{0}'.format(size))
            yield LinkMeta('shortcut-icon', type=icon.content_type, href=icon_url)  # pylint: disable=no-member


@view_config(name='favicon.ico', request_type=IPyAMSLayer)
def favorite_icon(request):
    """Favorite icon view"""
    configuration = IZMIConfiguration(request.context)
    if configuration.favicon is not None:
        request = request.copy()
        request.context = configuration.favicon
        return FileView(request)
    return HTTPNotFound()



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.skin module

Generic skin management components.
"""

from zope.interface import Interface, alsoProvides, implementer

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IForm, IGroup, IInnerSubForm
from pyams_form.subform import InnerEditForm
from pyams_layer.interfaces import IPyAMSLayer, ISkinnable, IUserSkinnable, MANAGE_SKIN_PERMISSION
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, FormGroupChecker
from pyams_zmi.helper.event import get_json_widget_refresh_callback
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_zmi import _


@viewlet_config(name='user-skin-properties.menu',
                context=IUserSkinnable, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=500,
                permission=MANAGE_SKIN_PERMISSION)
class UserSkinnablePropertiesMenuItem(NavigationMenuItem):
    """User skin properties menu"""

    label = _("Graphic theme")
    href = '#user-skin-properties.html'


class IUserSkinPropertiesForm(Interface):
    """User skin properties edit form marker interface"""


@ajax_form_config(name='user-skin-properties.html',
                  context=IUserSkinnable, layer=IPyAMSLayer,
                  permission=MANAGE_SKIN_PERMISSION)
class UserSkinPropertiesEditForm(AdminEditForm):
    """User skin properties edit form"""

    title = _("Graphic theme")

    def __init__(self, context, request):
        super().__init__(context, request)
        if not IUserSkinnable(context).can_inherit_skin:
            alsoProvides(self, IUserSkinPropertiesForm)


@adapter_config(name='skin-override',
                required=(IUserSkinnable, IAdminLayer, UserSkinPropertiesEditForm),
                provides=IGroup)
@implementer(IUserSkinPropertiesForm)
class UserSkinPropertiesInheritGroup(FormGroupChecker):
    """User skin properties inherit group"""

    def __new__(cls, context, request, parent_form):
        if not IUserSkinnable(context).can_inherit_skin:
            return None
        return FormGroupChecker.__new__(cls)

    fields = Fields(IUserSkinnable).select('override_skin')
    checker_fieldname = 'override_skin'
    checker_mode = 'disable'


@adapter_config(name='user-skin-properties',
                required=(IUserSkinnable, IAdminLayer, IUserSkinPropertiesForm),
                provides=IInnerSubForm)
class UserSkinPropertiesInnerEditForm(InnerEditForm):
    """User skin properties inner edit form"""

    @property
    def legend(self):
        if IUserSkinnable(self.context).can_inherit_skin:
            return None
        return _("Skin properties")

    border_class = ''

    fields = Fields(IUserSkinnable).select('skin', 'container_class', 'custom_stylesheet',
                                           'editor_stylesheet', 'custom_script')


@adapter_config(required=(IUserSkinnable, IAdminLayer, UserSkinPropertiesInnerEditForm),
                provides=IAJAXFormRenderer)
class UserSkinPropertiesInnerEditFormRenderer(ContextRequestViewAdapter):
    """User skin properties inner edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        callbacks = []
        if 'custom_stylesheet' in changes.get(ISkinnable, ()):
            callbacks.append(get_json_widget_refresh_callback(self.view, 'custom_stylesheet'))
        if 'editor_stylesheet' in changes.get(ISkinnable, ()):
            callbacks.append(get_json_widget_refresh_callback(self.view, 'editor_stylesheet'))
        if 'custom_script' in changes.get(ISkinnable, ()):
            callbacks.append(get_json_widget_refresh_callback(self.view, 'custom_script'))
        form = self.view.parent_form
        while not hasattr(form, 'success_message'):
            form = form.parent_form
        return {
            'status': 'success',
            'message': self.request.localizer.translate(form.success_message),
            'callbacks': callbacks
        }



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.utils module

This module provides a small set of generic components.
"""

from zope.component import queryAdapter, queryMultiAdapter
from zope.location import ILocation

from pyams_utils.adapter import adapter_config
from pyams_zmi.interfaces import IObjectLabel


__docformat__ = 'restructuredtext'


@adapter_config(required=ILocation, provides=IObjectLabel)
def location_label(context):
    """Basic location name factory"""
    return context.__name__


def get_object_label(context, request, view=None):
    """Get object label"""
    adapter = queryMultiAdapter((context, request, view), IObjectLabel)
    if adapter is None:
        adapter = queryAdapter(context, IObjectLabel)
    return adapter



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.timezone module

This modules defines views which are used to get access to PyAMS server timezone utility.
"""

from zope.interface import Interface

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import MANAGE_SYSTEM_PERMISSION
from pyams_utils.adapter import adapter_config
from pyams_utils.interfaces.timezone import IServerTimezone
from pyams_zmi.form import AdminModalEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.table import TableElementEditor


__docformat__ = 'restructuredtext'

from pyams_zmi import _


@adapter_config(required=(IServerTimezone, IAdminLayer, Interface),
                provides=ITableElementEditor)
class ServerTimezoneEditor(TableElementEditor):
    """Server timezone editor adapter"""


@ajax_form_config(name='properties.html', context=IServerTimezone, layer=IPyAMSLayer)
class ServerTimezonePropertiesEditForm(AdminModalEditForm):
    """Server timezone properties edit form"""

    title = _("Server timezone")
    legend = _("Properties")

    fields = Fields(IServerTimezone)

    _edit_permission = MANAGE_SYSTEM_PERMISSION



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.table module

This module provides bases classes for tables management.
"""

import json

from pyramid.decorator import reify
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_security.security import ProtectedViewObjectMixin
from pyams_table.column import Column, GetAttrColumn
from pyams_table.table import Table as BaseTable, get_weight
from pyams_template.template import get_view_template
from pyams_utils.adapter import ContextRequestViewAdapter
from pyams_utils.data import ObjectDataManagerMixin
from pyams_utils.date import SH_DATE_FORMAT, format_datetime
from pyams_utils.factory import get_object_factory, is_interface
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.list import boolean_iter
from pyams_utils.url import absolute_url
from pyams_zmi.interfaces.table import IInnerTable, IMultipleTableView, ITableAdminView, \
    ITableElementEditor
from pyams_zmi.utils import get_object_label
from pyams_zmi.view import InnerAdminView


__docformat__ = 'restructuredtext'

from pyams_zmi import _  # pylint: disable=ungrouped-imports


def get_table_id(table, context=None):
    """Table ID getter"""
    if context is None:
        context = table.context
    return '{}_{}'.format(table.prefix, ICacheKeyValue(context))


def get_column_sort(column, ignored=None):  # pylint: disable=unused-argument
    """Get column sortable attribute"""
    return getattr(column, 'sortable', None)


def get_column_type(column, ignored=None):  # pylint: disable=unused-argument
    """Get column sort type attribute"""
    return getattr(column, 'sort_type', None)


def get_row_id(table, element, context=None):
    """Row ID getter"""
    return '{}::{}'.format(get_table_id(table, context), ICacheKeyValue(element))


def get_row_name(element):
    """Row name getter"""
    return getattr(element, '__name__', None)


def get_row_editor(table, element):
    """Row editor getter"""
    return queryMultiAdapter((element, table.request, table), ITableElementEditor)


def check_attribute(attribute, source, column=None):
    """Check attribute value"""
    if callable(attribute):
        return attribute(source, column)
    return str(attribute)


def get_attributes(table, element, source, column=None):
    """Get table data attributes"""
    result = ''
    attrs = getattr(table, 'data_attributes', {}).get(element, {})
    for key, value in attrs.items():
        checked_value = check_attribute(value, source, column)
        if checked_value is not None:
            result += " {}='{}'".format(key, checked_value)
    return result


def get_data_attributes(element):
    """Get object data attributes"""
    data = IObjectData(element, None)
    if data is not None:
        return ' '.join(("data-{}='{}'".format(k, v if isinstance(v, str) else json.dumps(v))
                         for k, v in data.object_data.items()))  # pylint: disable=no-member
    return ''


def get_ordered_data_attributes(source, container, request, target='reorder.json'):
    """Get object data attributes for ordered table"""
    source.setdefault('table', {}).update({
        'data-searching': 'false',
        'data-info': 'false',
        'data-paging': 'false',
        'data-ordering': 'false',
        'data-row-reorder': '{"update": false}',
        'data-ams-location': absolute_url(container, request),
        'data-ams-reorder-url': target
    })
    source.setdefault('tr', {}).update({
        'data-ams-row-value': lambda row, col: get_row_name(row)
    })


class Table(ObjectDataManagerMixin, BaseTable):
    """Extended table class"""

    @reify
    def id(self):  # pylint: disable=invalid-name
        """Table ID getter"""
        return get_table_id(self, self.context)

    batch_size = 500
    start_batching_at = 500

    css_classes = {
        'table': 'table table-striped table-hover table-sm datatable'
    }

    object_data = {
        'responsive': True,
        'auto-width': False
    }

    @property
    def data_attributes(self):
        """Table data attributes getter

        These attributes are to be use with DataTables plug-in, and can be overridden in
        subclasses.
        """
        result = {
            'table': {
                'id': self.id,
                'data-ams-location': absolute_url(self.context, self.request)
            },
            'tr': {
                'id': lambda row, col: get_row_id(self, row),
                'data-ams-element-name': lambda row, col: get_row_name(row),
                'data-ams-url': lambda row, col: getattr(get_row_editor(self, row), 'href', None),
                'data-toggle':
                    lambda row, col: 'modal' if getattr(get_row_editor(self, row),
                                                        'modal_target', None)
                    else None
            },
            'th': {
                'data-ams-column-name': lambda row, col: row.__name__,
                'data-ams-sortable': get_column_sort,
                'data-ams-type': get_column_type
            }
        }
        return result

    def get_selected_row_class(self, row, css_class=None):
        """Get selected row class"""
        klass = self.css_classes.get('tr.selected')
        if callable(klass):
            klass = klass(*row)
        if klass and css_class:
            klass = '{} {}'.format(klass, css_class)
        else:
            klass = css_class or ''
        return klass

    def render_table(self):
        return super().render_table() \
            .replace('<table', '<table {}'.format(get_attributes(self, 'table', self))) \
            .replace('<table', '<table {}'.format(get_data_attributes(self)))

    def render_row(self, row, css_class=None):
        css_class = self.get_selected_row_class(row[0], css_class)
        return super().render_row(row, css_class) \
            .replace('<tr', '<tr {}'.format(get_attributes(self, 'tr', row[0][0])))

    def render_head_cell(self, column):
        return super().render_head_cell(column) \
            .replace('<th', '<th {}'.format(get_attributes(self, 'th', column))) \
            .replace('<th', '<th {}'.format(get_data_attributes(column)))

    def render_cell(self, item, column, colspan=0):
        return super().render_cell(item, column, colspan) \
            .replace('<td', '<td {}'.format(get_attributes(self, 'td', item, column)))


class InnerTableMixin:
    """Inner table mixin class"""

    table_class = Table
    table_label = FieldProperty(ITableAdminView['table_label'])

    empty_template = get_view_template(name='empty')

    def __init__(self, context, request, *args, **kwargs):
        super().__init__(context, request, *args, **kwargs)
        factory = get_object_factory(self.table_class) if is_interface(self.table_class) \
            else self.table_class
        self.table = factory(context, request)

    def update(self):
        """Admin view updater"""
        super().update()  # pylint: disable=no-member
        self.table.update()

    def render(self):
        """Admin view renderer"""
        has_values, values = boolean_iter(self.table.values)  # pylint: disable=no-member,unused-variable
        if (not has_values) and not getattr(self.table, 'display_if_empty', False):
            return self.empty_template()
        return super().render()  # pylint: disable=no-member


@implementer(ITableAdminView)
class TableAdminView(InnerTableMixin, InnerAdminView):
    """Table admin view

    This class is a wrapper for an admin view based on an inner table.
    """


@implementer(IInnerTable)
class InnerTableAdminView(InnerTableMixin):
    """Inner table admin view"""


class MultipleTablesMixin:
    """Multiple tables mixin view class"""

    table_label = FieldProperty(ITableAdminView['table_label'])

    @reify
    def tables(self):
        """Tables getter"""
        registry = self.request.registry
        return sorted((table
                       for name, table in
                       registry.getAdapters((self.context, self.request, self),  # pylint: disable=no-member
                                            IInnerTable)),
                      key=get_weight)

    def update(self):
        """View update"""
        super().update()
        for table in self.tables:
            table.update()


@implementer(IMultipleTableView)
class MultipleTablesAdminView(MultipleTablesMixin, InnerAdminView):
    """Multiple tables admin view"""


class I18nColumnMixin:
    """Column mixin with I18n header"""

    i18n_header = None

    @property
    def header(self):
        """Column header getter translation"""
        return self.request.localizer.translate(self.i18n_header)


class ReorderColumn(Column):
    """Reorder column"""

    weight = 0

    css_classes = {
        'th': 'reorder action',
        'td': 'action'
    }

    def render_cell(self, item):
        return '<i class="fas fa-arrows-alt-v"></i>'


class NameColumn(I18nColumnMixin, GetAttrColumn):
    """Common name column"""

    i18n_header = _("Name")
    weight = 10

    def get_value(self, obj):
        return get_object_label(obj, self.request, self.table)


class IconColumn(Column):
    """Base icon column"""

    header = ''
    css_classes = {
        'th': 'action'
    }
    sortable = 'false'

    icon_class = ''
    hint = ''

    permission = None
    checker = None

    def render_cell(self, item):
        """Column cell renderer based on permission and checker"""
        if not self.has_permission(item):
            return ''
        if self.checker:
            if callable(self.checker):
                checked = self.checker(item)  # pylint: disable=not-callable
            else:
                checked = self.checker
            if not checked:
                return ''
        return self.get_icon(item)

    def has_permission(self, item):
        """Column permission test"""
        if not self.permission:
            return True
        return self.request.has_permission(self.permission, context=item)

    def get_icon(self, item):
        """Column icon getter"""
        hint = self.get_icon_hint(item)
        return '<i class="fa-fw {} {}" data-original-title="{}"></i>'.format(
            self.get_icon_class(item), 'hint' if hint else '', hint)

    def get_icon_class(self, item):  # pylint: disable=unused-argument
        """Column class getter"""
        return self.icon_class

    def get_icon_hint(self, item):  # pylint: disable=unused-argument
        """Column hint getter"""
        return self.request.localizer.translate(self.hint)


class BaseActionColumn(IconColumn):
    """Base action column"""

    status = 'primary'
    href = None
    target = None
    modal_target = True

    permission = None

    css_classes = {
        'th': 'action',
        'td': 'action'
    }

    def is_visible(self, item):
        """Action visibility checker"""
        if not self.has_permission(item):
            return False
        if self.checker:
            if callable(self.checker):
                checked = self.checker(item)  # pylint: disable=not-callable
            else:
                checked = self.checker
            if not checked:
                return False
        return True

    def get_url(self, item):
        """Action URL getter"""
        return absolute_url(item, self.request, self.href)


class ActionColumn(ProtectedViewObjectMixin, BaseActionColumn):
    """Base action column

    This column class inherits from :py:class:`ProtectedViewObjectMixin
    <pyams_security.security.ProtectedViewObjectMixin>`, so you can use adapters
    to define permission required to enable this action.
    """

    def render_cell(self, item):
        """Column cell renderer"""
        if not self.is_visible(item):
            return ''
        return '<a href="{}" ' \
               '   data-ams-stop-propagation="true" ' \
               '   {} {} {}>{}</a>'.format(
                self.get_url(item),
                'data-ams-target="{}"'.format(self.target) if self.target else '',
                'data-toggle="modal"' if self.modal_target else '',
                'data-ams-modules="modal"' if self.modal_target else '',
                self.get_icon(item))


class ButtonColumn(BaseActionColumn):
    """Button action column"""

    label = None

    css_classes = {
        'th': 'action',
        'td': 'action py-1'
    }

    def render_cell(self, item):
        """Column cell renderer"""
        if not self.is_visible(item):
            return ''
        return '<a class="btn btn-sm btn-{} text-nowrap" ' \
               '   href="{}" ' \
               '   data-ams-stop-propagation="true" ' \
               '   {} {} {}>{}</a>'.format(
                self.status,
                self.get_url(item),
                'data-ams-target="{0}"'.format(self.target) if self.target else '',
                'data-toggle="modal"' if self.modal_target else '',
                'data-ams-modules="modal"' if self.modal_target else '',
                self.request.localizer.translate(self.label))


class JsActionColumn(ActionColumn):
    """Javascript action column"""

    def get_url(self, item):
        """Action URL getter"""
        return self.href


class TrashColumn(ObjectDataManagerMixin, JsActionColumn):
    """Trash column"""

    hint = _("Delete element")
    icon_class = 'fa fa-trash-alt'

    action_type = 'delete'

    href = 'MyAMS.container.deleteElement'
    modal_target = False

    object_data = {
        'ams-modules': 'container'
    }

    weight = 999


class DateColumn(GetAttrColumn):
    """Date or datetime column"""

    formatter = SH_DATE_FORMAT

    def get_value(self, obj):
        """Date column value getter"""
        value = super().get_value(obj)
        if not value:
            return '--'
        return format_datetime(value, self.formatter, self.request)


@implementer(ITableElementEditor)
class TableElementEditor(ContextRequestViewAdapter):
    """Base table element editor"""

    view_name = FieldProperty(ITableElementEditor['view_name'])
    modal_target = FieldProperty(ITableElementEditor['modal_target'])

    @property
    def href(self):
        """Table element editor getter"""
        return absolute_url(self.context, self.request, self.view_name)



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.controlpanel module

This module defines views and content providers which are used to get access to the local
site manager contents view.
"""

from zope.interface import implementer

from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import ITableWithActions
from pyams_zmi.interfaces.viewlet import IControlPanelMenu, IUtilitiesMenu
from pyams_zmi.table import NameColumn, Table, TableAdminView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_zmi import _


@viewletmanager_config(name='utilities.menu', context=ISiteRoot, layer=IAdminLayer,
                       manager=IControlPanelMenu, weight=10,
                       permission=VIEW_SYSTEM_PERMISSION, provides=IUtilitiesMenu)
class UtilitiesMenuItem(NavigationMenuItem):
    """Utilities menu item"""

    label = _("Utilities")
    icon_class = 'fab fa-codepen'
    href = '#utilities.html'


@implementer(ITableWithActions)
class UtilitiesTable(Table):
    """Utilities table list"""


@adapter_config(required=(ISiteRoot, IAdminLayer, UtilitiesTable),
                provides=IValues)
class UtilitiesTableValues(ContextRequestViewAdapter):
    """Utilities tables values adapter"""

    @property
    def values(self):
        """Utilities table values"""
        sm = self.context.getSiteManager()  # pylint: disable=invalid-name
        yield from sm.values()


@adapter_config(name='name',
                required=(ISiteRoot, IAdminLayer, UtilitiesTable),
                provides=IColumn)
class UtilityNameColumn(NameColumn):
    """Utility name column"""


@pagelet_config(name='utilities.html', context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class UtilitiesView(TableAdminView):
    """Utilities view"""

    title = _("Control panel")
    table_class = UtilitiesTable
    table_label = _("Site utilities")



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.viewlet.nav module

This module is used to define components which are used to build navigation menu.
"""

from pyams_template.template import template_config
from pyams_viewlet.manager import TemplateBasedViewletManager, WeightOrderedViewletManager, \
    viewletmanager_config
from pyams_viewlet.viewlet import Viewlet, viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.configuration import IZMIConfiguration
from pyams_zmi.interfaces.viewlet import ILeftAsideViewletManager, INavigationViewletManager, \
    IUserLinksViewletManager


__docformat__ = 'restructuredtext'


@viewletmanager_config(name='pyams.page_nav', layer=IAdminLayer,
                       manager=ILeftAsideViewletManager, weight=50,
                       provides=INavigationViewletManager)
@template_config(template='templates/page-nav.pt')
class NavigationViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Navigation viewlet manager"""


@viewlet_config(name='hide-nav', layer=IAdminLayer,
                manager=IUserLinksViewletManager, weight=999)
@template_config(template='templates/hide-nav.pt')
class HideNavigationViewlet(Viewlet):
    """Hide navigation viewlet"""

    def __new__(cls, context, request, view, manager):  # pylint: disable=unused-argument
        configuration = IZMIConfiguration(request.root)
        if (configuration is None) or not configuration.include_minify_button:
            return None
        return Viewlet.__new__(cls)



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.aside module

This module defines several content providers and viewlets which are used inside PyAMS
management interface navigation panel.
"""

from pyams_template.template import template_config
from pyams_viewlet.manager import TemplateBasedViewletManager, WeightOrderedViewletManager, \
    viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import ILeftAsideViewletManager


__docformat__ = 'restructuredtext'


@viewletmanager_config(name='pyams.page_aside', layer=IAdminLayer,
                       provides=ILeftAsideViewletManager)
@template_config(template='templates/page-aside.pt')
class LeftAsideViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Left aside viewlet manager"""


@viewlet_config(name='ajax-gear', layer=IAdminLayer,
                manager=ILeftAsideViewletManager, weight=1)
@template_config(template='templates/ajax-gear.pt')
class AJAXGearViewlet:
    """AJAX gear viewlet"""


@viewlet_config(name='version', layer=IAdminLayer,
                manager=ILeftAsideViewletManager, weight=999)
@template_config(template='templates/version.pt')
class VersionViewlet:
    """Version viewlet"""
